using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Data;
using System.ComponentModel;
using Microsoft.FamilyShowLib;
using System.Globalization;
using System.Collections;
using System.Windows.Controls;

namespace Microsoft.FamilyShow
{
    /// <summary>
    /// Interaction logic for SharedBirthdays.xaml
    /// </summary>

    public partial class SharedBirthdays : System.Windows.Controls.UserControl
    {
        private static ListCollectionView lcv;

        #region dependency properties

        public static readonly DependencyProperty PeopleCollectionProperty =
            DependencyProperty.Register("PeopleCollection", typeof(Object), typeof(SharedBirthdays),
                new FrameworkPropertyMetadata(new PropertyChangedCallback(PeopleCollectionProperty_Changed)));

        /// <summary>
        /// The Collection that will be used to build the Tag Cloud
        /// </summary>
        public Object PeopleCollection
        {
            get { return (Object)GetValue(PeopleCollectionProperty); }
            set { SetValue(PeopleCollectionProperty, value); }
        }

        #endregion

        #region routed events

        public static readonly RoutedEvent SharedBirthdaysSelectionChangedEvent = EventManager.RegisterRoutedEvent(
            "SharedBirthdaysSelectionChanged", RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(SharedBirthdays));

        public event RoutedEventHandler SharedBirthdaysSelectionChanged
        {
            add { AddHandler(SharedBirthdaysSelectionChangedEvent, value); }
            remove { RemoveHandler(SharedBirthdaysSelectionChangedEvent, value); }
        }

        #endregion

        public SharedBirthdays()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Used as a filter predicate to see if the person should be included 
        /// </summary>
        /// <param name="o">Person object</param>
        /// <returns>True if the person should be included in the filter, otherwise false</returns>
        public static bool FilterPerson(object o)
        {
            Person p = o as Person;
            return (p.BirthDate != null);
        }

        private static void PeopleCollectionProperty_Changed(DependencyObject sender, DependencyPropertyChangedEventArgs args)
        {
            SharedBirthdays sharedBirthdays = ((SharedBirthdays)sender);

            // ListCollectionView is used for sorting and grouping
            lcv = new ListCollectionView((IList)args.NewValue);

            // Include only those people with a birthdate
            lcv.Filter = new Predicate<object>(FilterPerson);

            // Sort by Month and Day only
            lcv.CustomSort = new MonthDayComparer();

            // Group the collection by the month and day of the person's birthdate
            lcv.GroupDescriptions.Add(new PropertyGroupDescription("BirthMonthAndDay"));

            sharedBirthdays.GroupedItemsControl.ItemsSource = lcv;
        }

        private void GroupedListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Person selected = ((ListBox)sender).SelectedItem as Person;

            if (selected != null)
                RaiseEvent(new RoutedEventArgs(SharedBirthdaysSelectionChangedEvent, selected.BirthDate));
        }

        internal static void Refresh()
        {
            lcv.Refresh();
        }

        internal void ClearSelection()
        {
            GroupedItemsControl.UnselectAll();
        }
    }

    /// <summary>
    /// Compare the birthdates (month and day) and first names of two people
    /// </summary>
    public class MonthDayComparer : IComparer
    {
        #region IComparer Members

        public int Compare(object x, object y)
        {
            Person p1 = x as Person;
            Person p2 = y as Person;

            if (p1 == p2)
                return 0;

            // Check the month first
            if (p1.BirthDate.Value.Month < p2.BirthDate.Value.Month)
                return -1;
            else if (p1.BirthDate.Value.Month > p2.BirthDate.Value.Month)
                return 1;
            else
            {
                // Since the months were the same, now check the day
                if (p1.BirthDate.Value.Day < p2.BirthDate.Value.Day)
                    return -1;
                else if (p1.BirthDate.Value.Day > p2.BirthDate.Value.Day)
                    return 1;
                else
                {
                    // The days are the same so check the first name
                    return (String.Compare(p1.FirstName, p2.FirstName, true, CultureInfo.CurrentCulture));
                }
            }
        }

        #endregion
    }
}